#!/usr/bin/env python

import cgi
import cgitb; cgitb.enable()
import time

class CFormulario:
	def __init__(self):
		"""Mapeo y control de errores en el servidor del formulario de entrada.

		@rtype: tupla.
		@return: objeto tupla.
		El primer elemento de la tupla indica si el formulario tiene cumplimentados los campos obligatorios con valores validos, 
			Si tupla[0]==False, tupla[1] es una cadena con un mensaje de error.
			Si tupla[0]==True, el resto de la tupla contiene los valores ingresados en los campos del formulario ordenados.
		"""
		self.error = []					# lista con los codigos de error detectados al parsear el contenido del formulario
		form = cgi.FieldStorage(keep_blank_values=1)	# parsea la peticion, form se comporta como un diccionario
		# "keep_blank_values=1" evita que FielStorage ignore aquellos campos que contengan cadenas vacias

		# Nombre Identificador de la CA	que va a emitir el certificado
		if form.has_key("idCA") and form["idCA"].value != "":
			self.idCA = form["idCA"].value
		else:
			self.error.append(1)		#"ERROR: Falta la informacion referente al Nombre Identificador de la CA."

		# Password de la CA
		if form.has_key("passwCA") and form["passwCA"].value != "":
			self.passwCA = form["passwCA"].value
		else:
			self.error.append(15)		# ERROR grave

		# Nombre Comun (CN)
		if form.has_key("comName") and form["comName"].value != "":
			self.cn = form["comName"].value
		else:
			self.error.append(2)		#ERROR

		# Apellido (SN)
		if form.has_key("surname") and form["surname"].value != "":
			self.sn = form["surname"].value
		else:
			self.error.append(16)		#ERROR

		# Sobrenombre (GN)
		self.gn = form["givName"].value

		# Numero de Serie
		self.serial = form["serial"].value

		# Organizacion (O)
		if form.has_key("org") and form["org"].value != "":
			self.o = form["org"].value
		else:
			self.error.append(3)		#ERROR: valor obligatorio

		# Unidad Organizativa (OU)
		self.ou = form["uniOrg"].value

		# Email
		if form.has_key("emaCont") and form["emaCont"].value != "":
			self.mail = form["emaCont"].value
		else:
			self.error.append(4)		#ERROR: valor obligatorio

		# Localidad
		if form.has_key("ciudad") and form["ciudad"].value != "":
			self.loc = form["ciudad"].value
		else:
			self.error.append(5)   		#ERROR: valor obligatorio

		# Provincia
		if form.has_key("provincia") and form["provincia"].value != "":
			self.st = form["provincia"].value
		else:
			self.error.append(6)   		#ERROR: valor obligatorio

		# Pais
		if form.has_key("pais") and form["pais"].value != "":
			self.pais = form["pais"].value
		else:
			self.error.append(7)   		#ERROR: valor obligatorio

		# Niveles en la cadena de certificados
		self.cadCert = form["numHijos"].value

		# Numero de Bits de la PrKey
		if form.has_key("numBits") and form["numBits"].value != "":
			opc = form["numBits"].value
			numBitsDic = {'0':'512', '1':'1024', '2':'2048', '3':'2060', '4':'2112'}
			self.numBits = numBitsDic[opc]
		else:
			self.numBits = '1024'			# valor por defecto

		# Exponente de Key
		if form.has_key("exponente") and form["exponente"].value != "":
			opcExp = form["exponente"].value
			expsDic = {'0':"RSA_3 (0x03L)", '1':"RSA_F4 (0x10001L)"}
			self.exp = expsDic[opcExp]
		else:
			self.exp = "RSA_F4 (0x10001L)"	# valor por defecto

		# Password
		if form.has_key("password") and form["password"].value != "":
			self.passw = form["password"].value
		else:
			self.error.append(8)		#"ERROR: Introduzca un password para encriptar su clave RSA."

		# Verificacion del password
		if form.has_key("vPassword") and form["vPassword"].value != "":
			vpassw = form["vPassword"].value
		else:
			self.error.append(9)		#"ERROR: Rellene la casilla de verificacion del password para encriptar su clave RSA."

		# Comprobando que coinciden las dos casillas de password
		if self.passw != vpassw:
			self.error.append(10)		#"ERROR: Las casillas Password y Verificacion de Password NO coinciden."

		# Fecha de Revocacion
		# FR-dia
		if form.has_key("fechaCaduc_1") and form["fechaCaduc_1"].value != "":
			self.frd = form["fechaCaduc_1"].value
		else:
			self.error.append(11)		#"ERROR: Seleccione el dia en la fecha de revocacion del certificado."
		# FR-mes
		if form.has_key("fechaCaduc_2") and form["fechaCaduc_2"].value != "":
			self.frm = form["fechaCaduc_2"].value
		else:
			self.error.append(12)		#"ERROR: Seleccione el mes en la fecha de revocacion del certificado."
		#FR-anyo
		if form.has_key("fechaCaduc_3") and form["fechaCaduc_3"].value != "":
			self.fra = form["fechaCaduc_3"].value
		else:
			self.error.append(13)		#"ERROR: Seleccione el anyo en la fecha de revocacion del certificado."

		# Comprobando que la fecha de caducidad es posterior a la actual
		fechas = compFechaAct (int(self.frd), int(self.frm), int(self.fra))
		if not fechas:
			self.error.append(14)		#"ERROR: La fecha introducida en el formulario no es posterior a la actual."

		# Comprobando que se ha marcado alguno de los usos del certificado
		# XXX XXX XXX


	def gestErrores (self):
		""" Devuelve una cadena enumerando los errores detectados en el formulario.
		"""
		msg = ""
		if len(self.error) != 0:
			msg += """<br/>Este mensaje de error ha sido generado por el servidor de la aplicacion, esto implica que existe algun problema con la verificacion de los datos en el cliente.<br/>

<br/>El problema puede deberse a varias causas, por favor compruebe que cumple los siguientes requerimientos:
	<blockquote><br/>1.- Asegurese de utilizar un navegador web que permita ejecutar codigo javascript.
	<br/>2.- No trate de realizar peticiones al servidor sin usar los formularios suministrados y no altere su codigo fuente.
	<br/>3.- Compruebe que ha rellenado correctamente el formulario, principalmente los campos marcados con *.</blockquote>

<br/>A continuacion se muestran los fallos detectados: 
"""
			for i in self.error:
				if i==1:
					msg += """ERROR (01NC): Debe rellenar el campo: "Autoridad de Certificacion (CA)" \n"""
				if i==2:
					msg += """ERROR (02NC): Debe rellenar el campo: "*Nombre Comun (CN):" \n"""
				if i==3:
					msg += """ERROR (03NC): Debe rellenar el campo: "*Organizacion (O)" \n"""
				if i==4:
					msg += """ERROR (04NC): Debe rellenar el campo: "*Email" \n"""
				if i==5:
					msg += """ERROR (05NC): Debe rellenar el campo: "*Datos de Localizacion -Ciudad" \n"""
				if i==6:
					msg += """ERROR (06NC): Debe rellenar el campo: "*Datos de Localizacion - Estado/Provincia/Region" \n"""
				if i==7:
					msg += """ERROR (07NC): Debe rellenar el campo: "*Datos de Localizacion - Pais" \n"""
				if i==8:
					msg += """ERROR (08NC): Debe rellenar el campo: "*Password" \n"""
				if i==9:
					msg += """ERROR (09NC): Debe rellenar el campo: "*Verificacion del Password" \n"""
				if i==10:
					msg += """ERROR (10NC): El contenido de los campos "*Password" y "*Verificacion del Password" NO coinciden." \n"""
				if i==11:
					msg += """ERROR (11NC): Debe rellenar el campo: "*Fecha de Caducidad - DD" \n"""
				if i==12:
					msg += """ERROR (12NC): Debe rellenar el campo: "*Fecha de Caducidad" - MM \n"""
				if i==13:
					msg += """ERROR (13NC): Debe rellenar el campo: "*Fecha de Caducidad" - AAAA \n"""
				if i==14:
					msg += """ERROR (14NC): La fecha introducida en el campo "*Fecha de Caducidad" debe ser posterior a la actual. \n"""
				if i==15:
					msg += """ERROR (15NC): Problema grave, no se ha podido obtener el password de la CA. \n"""
				if i==16:
					msg += """ERROR (16NC): Debe rellenar el campo: "*Apellido (SN):" \n"""
		return msg


	def vistaPrevia(self):
		"""Esta funcion es llamada unicamente en el caso en que el formulario y su contenido sean correctos.
		"""
		# generamos la vista previa html
		htmlCodeIni = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>Nuevo Certificado Digital</title>
	<link rel="stylesheet" type="text/css" href="../web/forms/view.css" media="all">
	<script type="text/javascript" src="../web/forms/view.js"></script>
	<script type="text/javascript" src="../web/forms/calendar.js"></script>
</head>
<body id="main_body" >
	<img id="top" src="../web/forms/top.png" alt="">
	<img src="../web/miarc.jpg" alt="miARC" border="0" usemap="#myARCLogoMap" id="myARCLogo">
	<map name="myARCLogoMap" id="myARCLogoMap"><area shape="rect" coords="21,2,243,59" href="../web/index.html" /></map>
	<div id="form_container">
		<br>
		<form id="form_43076" class="appnitro"  method="post" action="http://localhost:8000/cgi-bin/crearNuevoCert.py">
					<div class="form_description">
			<h2>Nuevo Certificado Digital</h2>
			<p>A continuacion se encuentra la informacion que ha ingresado para crear un nuevo Certificado Digital, compruebe que todos los datos son correctos.</p>
		</div>						
		<ul>
		<li id="li_1" >
			<label class="description" for="idCA">Autoridad de Certificacion (CA): </label>
			<div>
				<input type=hidden id="idCA" name="idCA" class="element text medium" type="text" maxlength="255" value='""" + self.idCA + """'/>

			</div><p class="guidelines" id="guide_1"><small>Autoridad de Certificacion que va a emitir el certificado.</small></p> 
		</li>

		<li id="li_20" >
			<label class="description" for="passwCA">Password de la Autoridad de Certificacion (CA): </label>
			<div>
				<input type=hidden id="passwCA" name="passwCA" class="element text medium" type="text" maxlength="255" value='""" + self.passwCA + """'/>

			</div>
		</li>

		<li class="section_break">
			<h3>Informacion del Cliente</h3>
			<p>Rellene los siguientes campos con la informacion previamente contrastada del cliente.</p>
		</li>
		<li id="li_2" >
			<label class="description" for="comName">*Nombre Comun (CN): </label>
			<div>
				<input type=hidden id="comName" name="comName" class="element text medium" type="text" maxlength="255" value='""" + self.cn + """'/>

			</div><p class="guidelines" id="guide_2"><small>Este campo se utiliza para identificar comunmente a la persona, empresa u organizacion que poseera el certificado.</small></p> 
		</li>"""

		snCode = """<li id="li_3" >
			<label class="description" for="surname">Apellido (SN): </label>
			<div>
				<input type=hidden id="surname" name="surname" class="element text medium" type="text" maxlength="255" value='""" + self.sn + """'/> 

			</div><p class="guidelines" id="guide_3"><small>Usese en el caso en que el cliente sea una persona.</small></p> 
		</li>"""

		gnCode = """<li id="li_4" >
			<label class="description" for="givName">Sobrenombre (GN): </label>
			<div>
				<input type=hidden id="givName" name="givName" class="element text medium" type="text" maxlength="255" value="'""" + self.gn + """'"/> 

			</div>
		</li>"""

		serialCode = """<li id="li_5" >
			<label class="description" for="serial">Numero de Serie: </label>
			<div>
				<input type=hidden id="serial" name="serial" class="element text medium" type="text" maxlength="255" value='""" + self.serial + """'/> 

			</div>
		</li>"""

		orgCode = """<li id="li_6" >
			<label class="description" for="org">*Organizacion (O): </label>
			<div>
				<input type=hidden id="org" name="org" class="element text large" type="text" maxlength="255" value='""" + self.o + """'/> 

			</div><p class="guidelines" id="guide_6"><small>Por ejemplo: el nombre de una compannia.</small></p>
		</li>"""

		uniOrgCode = """<li id="li_7" >
			<label class="description" for="uniOrg">Unidad Organizativa (OU): </label>
			<div>
				<input type=hidden id="uniOrg" name="uniOrg" class="element text large" type="text" maxlength="255" value='""" + self.ou + """'/> 

			</div><p class="guidelines" id="guide_7"><small>Por ejemplo: una seccion dentro de una compannia.</small></p>
		</li>"""

		emailCode = """<li id="li_8" >
			<label class="description" for="emaCont">*Email: </label>
			<div>
				<input type=hidden id="emaCont" name="emaCont" class="element text large" type="text" maxlength="255" value='""" + self.mail + """'/> 

			</div><p class="guidelines" id="guide_8"><small>Email de contacto.</small></p> 
		</li>"""

		locCode = """<li id="li_9" >
			<label class="description" for="loc">*Datos de Localizacion: </label>
			<div class="left">
				<input type=hidden id="ciudad" name="ciudad" class="element text medium" value='""" + self.loc + """' type="text">

				<label for="ciudad">Ciudad</label>
			</div>	
			<div class="right">
				<input type=hidden id="provincia" name="provincia" class="element text medium" value='""" + self.st + """' type="text">

				<label for="provincia">Estado / Provincia / Region</label>
			</div>
			<div class="left">
				<input type=hidden id="pais" name="pais" class="element text medium" value='""" + self.pais + """' type="text">

				<label for="pais">Pais</label>
			</div> 
		</li>"""

# XXX TRATAR USOS
		usosCode = """<li class="section_break">
			<h3>Usos del certificado:</h3>
			<p>Marque las casillas correspondientes con los usos que se le daran al Certificado Digital que se va a generar.</p>
		</li>
		<li id="li_16" >
			<label class="description" for="usos">Posibles Usos: </label>
			<span>
				<input type=hidden id="autorCert" name="autorCert" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="autorCert">Autoridad de Certificacion (Certification Authority Certificate)</label>
				<input type=hidden id="certSign" name="certSign" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="certSign">Firma de Certificados (Certificate Signing)</label>
				<input type=hidden id="objSign" name="objSign" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="objSign">Firma de Objetos (Objects Signing)</label>
				<input type=hidden id="SMIMEEnc" name="SMIMEEnc" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="SMIMEEnc">Encriptacion S/MIME (S/MIME Encryption)</label>
				<input type=hidden id="SMIMESign" name="SMIMESign" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="SMIMESign">Firma S/MIME (S/MIME Signing)</label>
				<input type=hidden id="SSLClient" name="SSLClient" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="SSLClient">Cliente SSL (SSL Client)</label>
				<input type=hidden id="SSLServer" name="SSLServer" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="SSLServer">Servidor SSL (SSL Server)</label>
				<input type=hidden id="serverAuth" name="serverAuth" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="serverAuth">Autentificacion de Servidor (Server Authentication)</label>
				<input type=hidden id="clientAuth" name="clientAuth" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="clientAuth">Autentificacion de Cliente (Client Authentication)</label>
				<input type=hidden id="CodeSign" name="CodeSign" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="CodeSign">Firma de Codigo (Code Signing)</label>
				<input type=hidden id="email" name="email" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="email">Email (Email)</label>
				<input type=hidden id="ipSecTrans" name="ipSecTrans" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="ipSecTrans">IPSec en modo transporte (IPSec End System)</label>
				<input type=hidden id="ipSecTunel" name="ipSecTunel" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="ipSecTunel">IPSec en modo tunel (IPSec Tunnel)</label>
				<input type=hidden id="ipSecUser" name="ipSecUser" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="ipSecUser">Usuario de IPSec (IPSec User)</label>
				<input type=hidden id="timeStamp" name="timeStamp" class="element checkbox" type="checkbox" value="1" />
				<label class="choice" for="timeStamp">Sellado en el Tiempo (Timestamping)</label>
		</span><p class="guidelines" id="guide_16"><small>Seleccione aquellos usos para los que sera destinado el certificado.</small></p> 
		</li>"""

		numEsl = """<li id="li_19" >
			<label class="description" for="numHijos">Longitud de la cadena de certificados: </label>
			<div>
				<input type=hidden id="numHijos" name="numHijos" class="element text small" type="text" maxlength="255" value=""/> 
			</div>
		</li>"""

		posCode = """<li class="section_break">
			<h3>Claves Publica y Privada.</h3>
			<p>Datos necesarios para generar el par de claves RSA asociadas al Certificado Digital.</p>
		</li>
		<li id="li_17" >
			<label class="description" for="numBits">Numero de bits de la Clave Privada: </label>
			<div>
				<input type=hidden id="numBits" name="numBits" class="element text small" type="text" maxlength="255" value='""" + self.numBits + """'/> 

			</div> 
		</li>
		<li id="li_18" >
			<label class="description" for="exp">Exponente: </label>
			<div>
				<input type=hidden id="exp" name="exp" class="element text small" type="text" maxlength="255" value='""" + self.exp + """'/> 

			</div>
		</li>
		<li id="li_12" >
			<label class="description" for="password">*Password: </label>
			<div>
				<input type=hidden id="password" name="password" class="element text large" type="password" maxlength="255" value='""" + self.passw + """'/> 

			</div>
		</li>
		<li class="section_break">
			<h3>Validez del Certificado</h3>
			<p>El certificado sera valido a partir del momento de emision hasta la fecha que usted elija a continuacion.
NOTA: La fecha debe ser anterior a la de revocacion del certificado raiz de la CA emisora.</p>
		</li>		
		<li id="li_15" >
			<label class="description" for="fechaCaduc">*Fecha de Caducidad: </label>
			<span>
				<input type=hidden id="fechaCaduc_1" name="fechaCaduc_1" class="element text" size="2" maxlength="2" value='""" + self.frd + """' type="text"/>
			</span>
			<span>
				<input type=hidden id="fechaCaduc_2" name="fechaCaduc_2" class="element text" size="2" maxlength="2" value='""" + self.frm + """' type="text"/>
			</span>
			<span>
		 		<input type=hidden id="fechaCaduc_3" name="fechaCaduc_3" class="element text" size="4" maxlength="4" value='""" + self.fra + """' type="text"/>
			</span>

			<p class="guidelines" id="guide_15"><small>Ultimo dia de validez del certificado.</small></p> 
		</li>		
		<li class="buttons">
		    <input type="hidden" name="form_id" value="43076" />
		    <input id="backForm" class="button_text" type="button" name="submit" value="Cambiar" onclick="history.back()"/>
			<input id="saveForm" class="button_text" type="submit" name="submit" value="Continuar"/>
		</li>
		</ul>
		</form>	
		<div id="footer">
			<p align="center">Creado por Francisco Jose Gonzalez Troyano para la Asignatura Proyecto Informatico de la <a href="http://www.informatica.us.es">ETSII</a></p>
		</div>
	</div>
	<img id="bottom" src="../web/forms/bottom.png" alt="">
</body>
</html>"""
		# Generacion de codigo a imprimir
		html = htmlCodeIni
		if self.sn != '':
			html += snCode
		if self.gn != '':
			html += gnCode
		if self.serial != '':
			html += serialCode
		html += orgCode
		if self.ou != '':
			html += uniOrgCode
		html += emailCode
		html += locCode
		html += usosCode
		html += numEsl
		html += posCode
		print html


def compFechaAct (dia, mes, anyo):
	""" Nos dice si la fecha pasada como parametro es posterior a la fecha actual.
		@param1type: entero
		@param2type: entero
		@param3type: entero
		@rtype: logico (boolean)
		@return: valor logico
	"""
	res = True
	tupTime = time.localtime()
	if anyo > tupTime.tm_year:
		res = True
	elif anyo < tupTime.tm_year:
		res = False
	else:
		if mes > tupTime.tm_mon:
			res = True
		elif mes < tupTime.tm_mon:
			res = False
		else:
			if dia > tupTime.tm_mday:
				res = True
			else:
				res = False
	return res


if __name__ == '__main__':
	print "Content-Type: text/html"     # sigue HTML
	print								# linea en blanco, fin de la cabecera
	cf = CFormulario()
	cadErrores = cf.gestErrores()
	if len(cadErrores)==0:
		cf.vistaPrevia()
	else:
		print cadErrores
