#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

import cgi
import cgitb; cgitb.enable()
import os
import os.path
from M2Crypto import RSA, m2, X509, EVP,Rand, Err
import time
import datetime
import extension
import sys

import lib.miarc_util
import lib.miarc_xml


class Certificado:

	# info by user
	passw = None				# pass to create RSA keys
	numDiasValidez = None		# número de dias de validez (int)
	fechaCaducidad = None		# fecha de caducidad (string)	

	# subject name
	cn = None					# common name
	sn = None					# surname
	gn = None					# given name
	serial = None				# serial
	o = None					# organization
	ou = None					# organization unit
	mail = None					# e-mail
	loc = None					# locality
	st = None					# state
	pais = None					# country

	# RSA
	numBits = None				# número de bits del módulo público (int)
	exp = None					# exponente público (long)

	# info by CA
	idCA = None					# id de la CA que firma el certificado
	numSer = None				# serial dado por la CA
	dirCA = None				# directorio la CA
	passwCA = None				# password de la CA para firmar el certificado
	tipoCA = False				# Boolean que indica si se trata de un certificado raiz de una CA para miARC

	# other
	rsa = None
	request = None
	cert = None
	uso = None
	cadCert = None

	def __init__(self):
		""" 
		Crea un certificado.
		"""
		self.capturarFormulario()

		# Si lo que estamos haciendo es rechazar una petición no se hará nada más
		if self.nuevoEstado != "rechazada":
			dirAct = os.getcwd()
			self.dirCA = dirAct + '/CAs/' + self.idCA + '/'
			dirCAPrivate = self.dirCA + 'private/'
			self.rsa = self.generarClaveRSA(self.numBits, self.exp)
			pkey = self.getClaveP(self.rsa)
			self.request = self.crearPeticion(pkey) 								# genera la petición

			# El certificado tiene que ser firmado con la clave privada de la CA.
			pPKey = RSA.load_key(dirCAPrivate + 'cakey.pem', self.web_passphrase_callback_CA)
			check = pPKey.check_key()
			if check == -1:
				print "ERROR: Ha habido un problema en la obtención del par de claves RSA de la CA. No se ha generado el certificado"
			caPkey = self.getClaveP(pPKey)
			self.cert = self.crearCert(self.request, caPkey, self.numDiasValidez) 	# genera el certificado
			self.salvarInfo()


	def salvarInfo(self):
		""" Salva en disco las claves, request (petición) y certificado digital
		"""
		numEnSerial = self.getSerial()
		numSerie = str(int(numEnSerial)-1)
		self.rsa.save_key(self.dirCA + 'private/' + numSerie + 'key.pem', 'aes_256_cbc', self.web_passphrase_callback) #des_ede3_cbc
		# cómo salvar la petición
		self.cert.save_pem(self.dirCA + 'certs/' + numSerie + '.pem')


	def capturarFormulario(self):
		"""
		Mapeo del formulario de entrada.
		"""
		form = cgi.FieldStorage(keep_blank_values=1)	# parsea la petición, form se comporta como un diccionario
		# "keep_blank_values = 1" evita que FielStorage ignore aquellos campos que contengan cadenas vacias			

		# si trata de gestionar una petición esta var contendrá el nuevo estado de la misma
		self.nuevoEstado = ""

		# Capturando el contenido del formulario sin errores
		# Campos requeridos:
		if form.has_key("idCA"):
			self.idCA = form["idCA"].value
		self.cn = form["comName"].value
		self.o = form["org"].value
		self.ou = form["uniOrg"].value
		self.mail = form["emaCont"].value
		self.loc = form["ciudad"].value
		self.st = form["provincia"].value
		self.pais = form["pais"].value
		self.uso = int(form["uso"].value)
		self.passw = form["password"].value
		self.numBits = int(form["numBits"].value)
		self.firma = form["firma"].value

		dia = form["fechaCaduc_1"].value
		if len (dia) == 1:
			dia = '0' + dia
		mes = form["fechaCaduc_2"].value
		if len (mes) == 1:
			mes = '0' + mes
		anyo = form["fechaCaduc_3"].value
		self.fechaCaducidad = dia + '-' + mes + '-' + anyo
		self.numDiasValidez = self.diasValidez(int(dia), int(mes), int(anyo))
		if form["exp"].value == "RSA_3 (0x03L)":
			self.exp = m2.RSA_3
		else:
			self.exp = m2.RSA_F4		

		# Campos optativos:
		if form.has_key("surname"):
			self.sn = form["surname"].value
		else:
			self.sn = ''
		if form.has_key("givName"):
			self.gn = form["givName"].value
		else:
			self.gn = ''
		if form.has_key("serial"):
			self.serial = form["serial"].value
		else:
			self.serial = ''
		
		if form.has_key("passwCA"):		# nuevo cert. a través del formulario de nuevo cert.
			self.passwCA = form["passwCA"].value

		# Parte añadida para gestión de peticiones.
		else:							# sabemos que se trata de una petición

			# Si el botón pulsado es el de aceptar petición
			try:
				if form["acepPet"].value == "Crear Certificado":
					# Obtener el passwCA del CAs.xml
					objCAXml = lib.miarc_xml.XMLCA()
					self.passwCA = objCAXml.getClave(self.idCA).encode('utf-8')
					self.nuevoEstado = 'aceptada'

			# Si el botón pulsado es el de rechazar petición
			except KeyError:
				if form["rechPet"].value == "Rechazar Petición":
					self.nuevoEstado = 'rechazada'

			# Parte correspondiente a la actualización del estado de la request en requests.xml
			self.idReq = form["idReq"].value
			dirAct = os.getcwd()
			pathReqXml = dirAct + '/requests/requests.xml'
			objReqXml = lib.miarc_xml.XMLReq(pathReqXml)
			objReqXml.cambiaEstado(self.idReq, self.nuevoEstado)
			# la petición ha sido aceptada
			# insertamos en requests.xml en el nodo <request> correspondiente los 
			# atributos idCA y sNCert que nos permitirán crear el enlace al 
			# certificado a crear
			objReqXml.addAttribute(self.idReq, 'idCA', self.idCA)
			f = open(dirAct + '/CAs/' + self.idCA + '/serial', 'r')
			sNCert = f.read()
			f.close()
			objReqXml.addAttribute(self.idReq, 'sNCert', sNCert)			


	def diasValidez (self, d, m, a):
		""" Devuelve el número de días que hay entre el día de generación del certificado 
		y la fecha de caducidad elegida en el formulario
		"""
		fechaRev = datetime.date(a, m, d)
		restaObj = fechaRev.__sub__(datetime.date.today())
		dias = restaObj.days
		return dias


	def web_passphrase_callback(self, p="", q=""): # argumentos 'p' y 'q' inútiles pero necesarios pq ya hemos comprobado que passw = vpassw, además  debido a que no se trata de una interface interactiva en tiempo real no se puede seguir la metodología empleada en RSA.py donde def save_key(self, file, cipher='aes_128_cbc', callback=util.passphrase_callback): y util.passphrase_callback requiere entrada de parámetros por consola
		""" Devuelve el password que se obtuvo del formulario.
			El control de diferencias en los passwords se realiza en controlForm().

			Se trata de una función necesaria porque openssl espera un objeto del tipo Python callable.
		"""
		return self.passw

	# Identico a la anterior pero tiene que capturar el password de la CA
	def web_passphrase_callback_CA(self, p="", q=""): # argumentos 'p' y 'q' inútiles pero necesarios pq ya hemos comprobado que passw = vpassw, además  debido a que no se trata de una interface interactiva en tiempo real no se puede seguir la metodología empleada en RSA.py donde def save_key(self, file, cipher='aes_128_cbc', callback=util.passphrase_callback): y util.passphrase_callback requiere entrada de parámetros por consola
		""" Devuelve el password que se obtuvo del formulario.
			El control de diferencias en los passwords se realiza en controlForm().

			Se trata de una función necesaria porque openssl espera un objeto del tipo Python callable.
		"""
		return self.passwCA


	def generarClaveRSA(self, pkb = 1024, e = m2.RSA_F4):
		"""
		Genera un nuevo par de claves RSA.

		@type pkb: int
		@param pkb: especifica el número de bits en el módulo público, 
		minimo 1024, recomendables 2048.
		@type e: hex long
		@param e: valor a utilizar por el exponente público, 
		recomendable utilizar las constantes openSSL RSA_3 (0x03L) o 
		RSA_F4 

		@rtype: M2Crypto.RSA.RSA
		@return: M2Crypto.RSA.RSA object.
		"""
		error = False
		rsaKeys = RSA.gen_key(pkb, e)
		if rsaKeys == 0:
			print "Error: La clave RSA generada no es correcta. Generando otra..."
		while rsaKeys == 0 or error == True:
			rsaKeys = RSA.gen_key(pkb, e)
		check = rsaKeys.check_key()
		if check == -1:
			print "Error: Ha habido un problema durante la comprobacion de las claves RSA. Generando otras..."	
			error = True
		elif check == 0:
			print "Error: Existe un problema con las claves RSA. Generandolas de nuevo..."
			error = True
		return rsaKeys


	def getClaveP(self, key):
		"""Recibe un par de claves RSA y devuelve la componente pública.

		   @type key: M2Crypto.RSA.RSA
		   @param key: par de claves RSA 

		   @rtype: M2Crypto.EVP.Pkey
		   @return: M2Crypto.EVP.Pkey object.
		"""
		pkey = EVP.PKey()
		pkey.assign_rsa(key)
		return pkey


	def crearPeticion(self, pkey):
		"""Crea una petición de certificado X.509 (X.509 Certificate Request).
	
		   @type pkey: M2Crypto.EVP.Pkey
		   @param pkey: componente pública de par de claves RSA 
		"""

		req = X509.Request()
		req.set_pubkey(pkey)
		name = X509.X509_Name()
		
		# Registro del subject name (para identificar al receptor del certificado)
		name.CN = self.cn
		name.SN = self.sn
		name.GN = self.gn
		name.C = self.pais
		name.ST = self.st
		name.L = self.loc
		name.O = self.o
		name.OU = self.ou
		name.Email = self.mail
		name.serialNumber = self.serial
		
		req.set_subject_name(name)

		# Gestion de extensiones en la solicitud del certificado
		ext1 = X509.new_extension('basicConstraints', 'CA:false')
		extstack = X509.X509_Extension_Stack()
		extstack.push(ext1)
		# Comprobacion del contenido de la pila
		assert(extstack[0].get_name() == 'basicConstraints')
		req.add_extensions(extstack)

		req.sign(pkey, self.firma)			# Firma de la petición
		return req


	def tratSerial (self):
		""" 
		Acceso de lectura e incremento del contenido del archivo serial.
		"""
		serial = open(self.dirCA + "serial", 'r+w')
		num = int(serial.read())
		numInc = num + 1
		serial.seek(0, 0)
		serial.write(str(numInc))
		serial.close()
		return num


	def getSerial (self):
		""" 
		Devuelve una cadena con el contenido del archivo serial.
		"""
		serial = open(self.dirCA + "serial", 'r')
		num = serial.read()
		serial.close()
		return num


	def crearCert(self, req, caPkey, dias):
		"""Crea el certificado a partir del certifique request, la clave RSA y 
		   el número de días de validez firmado por la CA.
		"""
		pkey = req.get_pubkey()
		if not req.verify(pkey):
			raise ValueError, "Error verificando la clave pública de la petición."
		sub = req.get_subject()

		cert = X509.X509()
		num = self.tratSerial()
		self.numSer = num			# almacenamos el número de serie que le corresponde al objeto self
		cert.set_serial_number(num)
		cert.set_version(2)
		cert.set_subject(sub)
		# Incluyendo el DN de la CA emisora en el certificado
		certCA = X509.load_cert(self.dirCA + 'cacert.pem')
		issuer = certCA.get_subject()
		cert.set_issuer(issuer)

		cert.set_pubkey(pkey)
		notBefore = m2.x509_get_not_before(cert.x509)
		notAfter  = m2.x509_get_not_after(cert.x509)
		m2.x509_gmtime_adj(notBefore, 0)
		m2.x509_gmtime_adj(notAfter, 60*60*24*dias) 	# Período de validez

		# Gestión de extensiones (selección no manual de extens. en la interfaz):
		extension.PlantillaUso(self.uso, cert)


# XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX
#		# el tercer argumento a 1 indica que la extensión es crítica
#		bc = extension.BasicConstraint(False, -1, True)
#
#		usosPermitidosKU = ['keyEncipherment', 'dataEncipherment', 'keyAgreement', 'keyCertSign']
#		ku = extension.KeyUsage(usosPermitidosKU, False)
#
#		usosPermitidosEKU = ['timeStamping', 'msCodeInd', 'msCodeCom', 'msCTLSign',	'msSGC']
#		eku = extension.ExtendedKeyUsage(usosPermitidosEKU, False)
#
#		ski = extension.SubjectKeyIdentifier('32eb', True)
#
## XXX
##		aki = extension.AuthorityKeyIdentifier('keyid, issuer', True)
## XXX
#
#		san = extension.SubjectAlternativeName(True)
#
#		iap = extension.InhibitAnyPolicy(3, True)
#
#		nc = extension.NameConstraints('opciones', True)
#
#		urls = ['my.com/my.crl', 'oth.com/my.crl']
#		crldp = extension.CRLDistributionPoints(urls, True)
#
#		valoresPC = {'requireExplicitPolicy':4, 'inhibitPolicyMapping':1}
#		pc = extension.PolicyConstraints(valoresPC, True)
#
#		subcategoria = 7
#		valor = "comentarioOOOOO"
#		nse = extension.NetscapeStringExtension(subcategoria, valor)
#
#		nct = extension.NetscapeCertificateType()
#
#		cert.add_ext(bc.ext)
#		cert.add_ext(ku.ext)
#		cert.add_ext(eku.ext)
#		cert.add_ext(ski.ext)
##		cert.add_ext(aki.ext)
#		cert.add_ext(san.ext)
#		cert.add_ext(iap.ext)
#		cert.add_ext(nc.ext)
#		cert.add_ext(crldp.ext)
#		cert.add_ext(pc.ext)
#		cert.add_ext(nse.ext)
#		cert.add_ext(nct.ext)
# XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX


		cert.sign(caPkey, self.firma)
		return cert



# Las creaciones o actualizaciones de archivos XML no meterlos en la clase Certificado
def actualizaCertsXML(cert):
	""" 
	Actualización del contenido del archivo certs.xml almacenando todos 
	los campos del subject name del certificado pasado como argumento.
	"""

	# generando la cadena con la fecha actual para el atributo fechaUltRevCad
	fechaActString = lib.miarc_util.fechaActCad()

	dirAct = os.getcwd()
	pathXml = dirAct + '/web/CAs/' + cert.idCA + '/certs.xml'
	if not os.path.exists(pathXml):
		fXml = open (pathXml, 'w')		# el archivo certs.xml ya debería estar creado
		etiquetaRaiz = """<?xml version="1.0" encoding="UTF-8" ?>\n<CA id='""" + cert.idCA + """' fechaUltRevCad = '""" + fechaActString + """'>\n</CA>"""
		fXml.write(etiquetaRaiz)
	else:
		fXml = open (pathXml, 'r+w')		# abre el ya existente certs.xml
		fXml.seek(-5, 2)					# desplazamos justo antes de </CA>
	lSNCert = generaLSNCert(cert)			# lista de cadenas con las filas de la estructura XML del certificado
	lSNCert.append("\n</CA>")				# restauramos la etiqueta de cierre del elemento raíz
	fXml.writelines(lSNCert)
	fXml.close()
	

# XXX Podría añadir más campos para permitir más posibilidades de búsqueda
# XXX por ej.: fecha de emisión y caducidad, numBits, exponente (estudiar posibilidades)
def generaLSNCert (cert):
	""" 
	Recibe un certificado digital como argumento y genera una lista de 
	cadenas que conforman una estructura XML válida a partir del Subject 
	Name del certificado.
	"""
	lSNCert = []

	raizInicNodo = "\n\t<cert numSer='" + str(cert.numSer) + "'>"
	raizFinNodo = "\n\t</cert>"

	cnNodo = "\n\t\t<cn>" + cert.cn + "</cn>"
	snNodo = "\n\t\t<sn>" + cert.sn + "</sn>"
	gnNodo = "\n\t\t<gn>" + cert.gn + "</gn>"
	paisNodo = "\n\t\t<pais>" + cert.pais + "</pais>"
	stNodo = "\n\t\t<st>" + cert.st + "</st>"
	locNodo = "\n\t\t<loc>" + cert.loc + "</loc>"
	oNodo = "\n\t\t<o>" + cert.o + "</o>"
	ouNodo = "\n\t\t<ou>" + cert.ou + "</ou>"
	emailNodo = "\n\t\t<email>" + cert.mail + "</email>"
	serialNodo = "\n\t\t<serial>" + cert.serial + "</serial>"
	extNodo = "\n\t\t<ext>" + "PENDIENTE DE IMPLEMENTACION (EXTENSIONES)" + "</ext>"
	diasValNodo = "\n\t\t<diasVal>" + str(cert.numDiasValidez) + "</diasVal>"
	
	lSNCert.append(raizInicNodo)
	lSNCert.append(cnNodo)
	lSNCert.append(snNodo)
	lSNCert.append(gnNodo)
	lSNCert.append(paisNodo)
	lSNCert.append(stNodo)
	lSNCert.append(locNodo)
	lSNCert.append(oNodo)
	lSNCert.append(ouNodo)
	lSNCert.append(emailNodo)
	lSNCert.append(serialNodo)
	lSNCert.append(extNodo)
	lSNCert.append(diasValNodo)
	lSNCert.append(raizFinNodo)

	return lSNCert


#	Debemos actualizar /CAs/idCA/gestCerts.html
def actualizaIndex (self):
	""" Actualiza el archivo html añadiendo la información correspondiente a la nueva CA.
	"""
	nuevaInfo = unicode("""
		<li>
		<table width="100%" border="0" align="center" cellpadding="0" cellspacing="0" id="idCA">
		<tbody>
		<tr>
			<td width="12%" class="imagen"><a href='./CAs/""" + self.idCA + """/""" + self.idCA + """.html'><img src="../web/CAs/candado.png" alt="" title="" border="0" height="60" width="60"></a></td>
			<td width="40%" class="preview"><p><strong>idCA: </strong>""" + self.idCA + """<br />
			  <strong>Nombre: </strong>""" + self.cn + """<br />
			  <strong>Organización: </strong>""" + self.o + """<br />
			  <strong>Fecha de Revocación: </strong>""" + self.frd + '/' + self.frm + '/' + self.fra + """</p></td>
			<td width="48%" class="enlace">
				<a href="./CAs/""" + self.idCA + """/formNC.html" class=""><img src="../web/images/icons/script_add.png" width="16" height="16" /> Emitir Nuevo Certificado</a><br/>
				<a href="./CAs/""" + self.idCA + """/revCert.html" class=""><img src="../web/images/icons/script_delete.png" width="16" height="16" /> Revocar Certificado</a><br/>
				<a href="./CAs/""" + self.idCA + """/crl.html" class=""><img src="../web/images/icons/script_go.png" width="16" height="16" /> Obtener Lista de Revocación de Certificados</a>
		</tr>
		</tbody>
		</table>
		</li>
		<div class="section_break"></div>
	""", 'utf-8')
	dirAct = os.getcwd()
	path = dirAct + '/web/index.html'
	lon = len(path) - 139 # 139 son los bytes que hay desde el final del archivo hasta donde queremos añadir el <li>
	if os.access(path, os.F_OK):
		index = open(path, 'r')
		contenido = index.read()
		index.close()
		pre = contenido[:lon]
		pos = contenido[lon+1:]
		nuevoContenido = pre + nuevaInfo.encode("utf-8") + pos
		index = open(path, 'w')
		index.write(nuevoContenido)
		index.close()
	else:
		print "ERROR: No se encuentra el archivo index.txt en el directorio web/"


#	Crearemos ahora /web/CAs/idCA/certs/numSerieCert.html
def crearCertHtml (cert):
	# directorio donde guadaremos numSerie.html (numSerie = *int* número de serie del certificado)
	dirAct = os.getcwd()
	pathDir = dirAct + '/web/CAs/' + cert.idCA + '/certs/'
	if not os.access (pathDir, os.F_OK):	
		os.mkdir(pathDir)
	# XXX
	numEnSerial = cert.getSerial()
	numSerie = str(int(numEnSerial)-1)
	# XXX
	f = open(pathDir + numSerie + '.html', 'w') # creamos el archivo

	# dando formato html al certificado
	certificado = cert.cert.as_text()

	contenido = unicode("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>miARC - Configuración del Certificado con Número de Serie: """ + numSerie + """ Emitido por la CA: """ + cert.idCA + """.</title>
	<link rel="stylesheet" type="text/css" href="../../../estiloInd.css" media="all">
	<script type="text/javascript" src="../../../view.js"></script>
	<style type="text/css">
		lis {clear:both; display:table;}
		span {float:left; padding: 0 46px 0 45px;}
	</style>
	<script type="text/javascript">
	<!--
	function seleccion(opc)
	{
		//escondemos todos los contenidos
		document.getElementById("op1").style.display="none";
		document.getElementById("op2").style.display="none";
		switch(opc)
		{
			case 1:
				//mostramos el contenido de la primera opción
				document.getElementById("op1").style.display="block";
				break;
			case 2:
				//mostramos el contenido de la segunda opción
				document.getElementById("op2").style.display="block";
				break;
		}
	}
	//-->
	</script>
	<script>
		function mostrar(enla) {
		  obj = document.getElementById('oculto');
		  obj.style.visibility = (obj.style.visibility == 'hidden') ? 'visible' : 'hidden';
		  enla.innerHTML = (enla.innerHTML == 'Ocultar Password') ? 'Mostrar Password' : 'Ocultar Password';
		}
	</script>
</head>
<body id="main_body" >
	<img id="top" src="../../../images/top.png" alt="">
	<img src="../../../images/miarc.jpg" alt="miARC" border="0" usemap="#myARCLogoMap" id="myARCLogo">
	<map name="myARCLogoMap" id="myARCLogoMap"><area shape="rect" coords="21,2,243,59" href="../../../index.html" /></map>
	<div id="contenedor">
		<br>
		<form id="form_39443" class="appnitro">
		<div class="form_description">
			<h2>Configuración del certificado con número de serie: """ + numSerie + """.</h2>
			<p>Desde esta página puede consultar los valores que fueron seleccionados por el operador durante el proceso de creación del certificado con <B>número de serie: """ + numSerie + """</B> emitido por la <B>autoridad de certificación: """ + cert.idCA + """</B> además del propio certificado en formato texto.</p>
		</div>
		<ul>
		<br/>
		<lis>
			<span style="border-right:1px solid;"><a href='javascript:seleccion(1);'><img src="../../../images/icons/application_form_magnify.png" width="16" height="16" /> Ver Configuración</a></span>
			<span style="border-right:1px solid;"><a href='javascript:seleccion(2);'><img src="../../../images/icons/script_key.png" width="16" height="16" /> Ver Certificado</a></span>
			<span><a href='javascript:seleccion(3);'><img src="../../../images/icons/magifier_zoom_out.png" width="16" height="16" /> Ocultar</a></span>
		</lis>
		<lis id="op1" style="display:block;width:99%;height:100%;"><br/><br/>
			""" + vistaCert(cert) + """
		</lis>
		<pre>
		<lis id="op2" style="display:none;width:99%;height:100%;"><br/>""" + certificado + """
		</lis>
		</pre>
		<lis id="op3" style="display:none;width:99%;height:100%;">
			Esta opción oculta la tabla
		</lis>
		<div class="section_break"></div>
		<table width="100%" border="0" align="center">
  		<tr>
			<td><div align="center">
				<a href='../../../../CAs/""" + cert.idCA + """/certs/""" + numSerie + """.pem' class=""><img src="../../../images/icons/script_save.png" width="16" height="16" /> Descargar Certificado</a>
			</div></td>
			<td><div align="center">
				<a href='../../../../CAs/""" + cert.idCA + """/private/""" + numSerie + """key.pem' class=""><img src="../../../images/icons/key.png" width="16" height="16" /> Descargar Clave Privada</a>
			</div></td>
  		</tr>
</table>
		<div class="section_break"></div>
		</ul>
		</form>
		<div id="footer">
			<p align="center">Creado por Francisco José González Troyano para la Asignatura Proyecto Informático de la <a href="http://www.informatica.us.es">ETSII</a></p>
		</div>
	</div>
	<img id="bottom" src="../../../images/bottom.png" alt="">
</body>
</html>""", 'utf-8')
	f.write(contenido.encode("utf-8"))
	f.close()


def vistaCert(cert):
	# SIMILAR A LA VISTA PREVIA
	pre = """
	<div class="section_break">
		<h3><u>Configuración del certificado.</u></h3>
		<div><p>Información suministrada por el operador utilizada para la creación del certificado.</p></div>
	</div>
	<li id="li_1" >
		<label class="description" for="idCA">Nombre Identificador de la CA emisora: </label>
		<div>
			""" + cert.idCA + """
		</div>
	</li><br>
	<div class="section_break">
		<h3>Datos relativos al propietario del certificado:</h3>
		<p>Información correspondiente al propietario del certificado.</p>
	</div>"""

	cnCode = """
	<li id="li_3" >
		<label class="description" for="nombreCom">Nombre Común (CN): </label>
		<div>
			""" + cert.cn + """
		</div>
	</li>"""

	snCode = """
	<li id="li_4" >
		<label class="description" for="apellido">Apellido (SN): </label>
		<div>
			""" + cert.sn + """
		</div>
	</li>"""

	gnCode = """
	<li id="li_5" >
		<label class="description" for="sobrenombre">Sobrenombre (GN): </label>
		<div>
			""" + cert.gn + """
		</div>
	</li>"""

	nsCode = """
	<li id="li_9" >
		<label class="description" for="serial">Número de Serie: </label>
		<div>
			""" + cert.serial + """
		</div>
	</li>"""

	oCode = """
	<li id="li_6" >
		<label class="description" for="organizacion">Organización (O): </label>
		<div>
			""" + cert.o + """
		</div>
	</li>"""

	ouCode = """
	<li id="li_7" >
		<label class="description" for="unidadOrg">Unidad Organizativa (OU): </label>
		<div>
			""" + cert.ou + """
		</div>
	</li>"""

	emailCode = """
	<li id="li_10" >
		<label class="description" for="email">E-mail: </label>
		<div>
			""" + cert.mail + """
		</div>
	</li>"""

	dirCode = """
	<li id="li_8" >
		<label class="description" for="direccion">Datos de Localización: </label>
	</li>"""

	locCode = """
	<li id="li_10" >
		<label class="description" for="direccion_3">Localidad: </label>
		<div>
			""" + cert.loc + """
		</div>
	</li>"""

	estCode = """
	<li id="li_10" >
		<label class="description" for="direccion_4">Estado / Provincia / Región: </label>
		<div>
			""" + cert.st + """
		</div>
	</li>"""

	paisCode = """
	<li id="li_10" >
		<label class="description" for="direccion_6">País: </label>
		<div>
			""" + cert.pais + """
		</div>
	</li>"""

	# Codificando los valores 'int' del campo "uso" a su nombre en la interfaz.
	if cert.uso == 1:
		opcUso = "S/MIME"
	elif cert.uso == 2:
		opcUso = "Cifrado"
	elif cert.uso == 3:
		opcUso = "Cliente SSL"
	elif cert.uso == 4:
		opcUso = "Servidor SSL"
	elif cert.uso == 5:
		opcUso = "Certificado de CA"
	elif cert.uso == 6:
		opcUso = "Firma Código/Objeto"

	pos = """
	<br><div class="section_break">
		<h3>Información referente a las opciones de creación del certificado de la CA.</h3>
		<p>A continuación se muestran las opciones elegidas durante el proceso de creación del certificado.</p>
	</div>
	<li id="li_12" >
		<label class="description" for="numBitsPK">Número de bits de la Clave Privada: </label>
		<div>
			""" + str(cert.numBits) + """
		</div>
	</li>
	<li id="li_13" >
		<label class="description" for="exponente">Exponente: </label>
		<div>
			""" + str(cert.exp) + """
		</div>
	</li>
	<li id="li_14" >				
		<label class="description" for="password">Password: </label>
		Por razones de seguridad el password se mantiene oculto por defecto, si desea verlo pulse sobre el siguiente enlace: <a href="#" onclick="mostrar(this); return false" />Mostrar Password</a>.
		<div id="oculto" style="visibility:hidden">
			Password: """ + str(cert.passw) + """
		</div>
	</li>
	<li id="li_25" >
		<label class="description" for="firma">Algoritmo de Firma: </label>
		<div>
			""" + str(cert.firma) + """
		</div>
	</li>
	<li id="li_25" >
		<label class="description" for="firma">Configuración de extensiones (Uso): </label>
		<div>
			""" + opcUso + """
		</div>
	</li><br>
	<div class="section_break">
		<h3>Fecha de caducidad.</h3>
		<p>El certificado será funcional hasta la siguiente fecha.</p>
	</div>
	<li id="li_17" >
		<label class="description" for="fechaCaduc">Fecha de Caducidad: </label>
		<div>
			""" + str(cert.fechaCaducidad) + """
		</div>
	</li>"""

	vista = pre
	if cert.cn != '':
		vista += cnCode
	if cert.sn != '':
		vista += snCode
	if cert.gn != '':
		vista += gnCode
	if cert.serial != '':
		vista += nsCode
	if cert.o != '':
		vista += oCode
	if cert.ou != '':
		vista += ouCode
	if cert.mail != '':
		vista += emailCode
	if cert.loc != '' or self.st != '' or self.pais != '':
		vista += dirCode
	if cert.loc != '':
		vista += locCode
	if cert.st != '':
		vista += estCode
	if cert.pais != '':
		vista += paisCode
	vista += pos
	return vista


def generaRespHtml(objCert):
	""" Pagina html que se mostrará tras enviar el formulario del certificado.
	"""
	# dando formato html al certificado
	certTexto = objCert.cert.as_text()
	numEnSerial = objCert.getSerial()
	numSerie = str(int(numEnSerial)-1)
	html = unicode("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>miARC - Nuevo Certificado Digital emitido con éxito.</title>
	<link rel="stylesheet" type="text/css" href="../web/estiloInd.css" media="all">
	<script type="text/javascript" src="../web/view.js"></script>
</head>
<body id="main_body" >
	<img id="top" src="../web/images/top.png" alt="">
	<img src="../web/images/miarc.jpg" alt="miARC" border="0" usemap="#myARCLogoMap" id="myARCLogo">
	<map name="myARCLogoMap" id="myARCLogoMap"><area shape="rect" coords="21,2,243,59" href="../web/index.html" /></map>
	<div id="contenedor">
		<br>
		<form id="form_39443" class="appnitro">
		<div class="form_description">
			<h2>Certificado Digital Creado con Éxito.</h2>
			<p>A continuación puede ver el certificado digital en formato texto, en la parte iniferior dispone de enlaces de descarga para el mismo y para su clave privada RSA.</p>
		</div>
		<ul>
			<table width="100%" border="0" align="center" cellpadding="0" cellspacing="0" id="idCA">
			<tbody>
			<tr>""", 'utf-8')
	if certif.nuevoEstado == "aceptada":
		html += unicode("""\n\t\t\t<td><div align="left"><a href="../web/tratReq.html"><img src="../web/images/icons/table_edit.png" width="16" height="16" /> Volver a Gestión de Peticiones</a></div></td>""", 'utf-8')
	else:
		html += unicode("""\n\t\t\t<td><div align="left"><a href="../web/CAs/""" + objCert.idCA + """/gestCerts.html"><img src="../web/images/icons/script.png" width="16" height="16" /> Volver a Gestión de Certificados</a></div></td>""", 'utf-8')

	html += unicode("""\n\t\t\t<td><div align="right"><a href='../web/CAs/""" + objCert.idCA + """/certs/""" + numSerie + """.html'/>Ir a Configuración del Certificado <img src="../web/images/icons/application_form_magnify.png" width="16" height="16" /></a></div></td>
				</tr>
			</tbody>
			</table>
		<div class="section_break"></div>
		<li>
			<pre>""" + certTexto + """</pre>
		</li>
		<div class="section_break"></div>
		<table width="100%" border="0" align="center">
		<tbody>
 		<tr>
			<td><div align="center"><a href='../../../CAs/""" + objCert.idCA + """/certs/""" + numSerie + """.pem' class=""><img src="../web/images/icons/script_save.png" width="16" height="16" /> Descargar Certificado Digital</a></div></td>
    		<td><div align="center"><a href='../../../CAs/""" + objCert.idCA + """/private/""" + numSerie + """key.pem' class=""><img src="../web/images/icons/key.png" width="16" height="16" /> Descargar Clave Privada</a></div></td>
 		</tr>
		</tbody>
		</table>
		<div class="section_break"></div>
		</ul>
		</form>
		<div id="footer">
			<p align="center">Creado por Francisco José González Troyano para la Asignatura Proyecto Informático de la <a href="http://www.informatica.us.es">ETSII</a></p>
		</div>
	</div>
	<img id="bottom" src="../web/images/bottom.png" alt="">
</body>
</html>""", 'utf-8')
	print html.encode("utf-8")


def generaRechPetHtml(idReq):
	""" Página html que se muestra si se rechaza una petición.
	"""
	htmlCode = unicode("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>miARC - Sistema de Creación y Gestión de Autoridades de Certificación</title>
	<link rel="stylesheet" type="text/css" href="../web/estiloInd.css" media="all">
	<link rel="shortcut icon" href="../web/favicon.ico">
	<script type="text/javascript" src="../web/view.js"></script>
</head>
<body id="main_body">
	<img id="top" src="../web/images/top.png" alt="">	
	<img src="../web/images/miarc.jpg" alt="miARC" border="0" usemap="#myARCLogoMap" id="myARCLogo">
	<map name="myARCLogoMap" id="myARCLogoMap"><area shape="rect" coords="21,2,243,59" href="../web/index.html" /></map>
	<div id="contenedor">
		<br />
		<form id="form_39443" class="appnitro">
		<div class="form_description">
			<h2>Petición """ + str(idReq.encode('utf-8')) + """ rechazada.</h2>
			<p>Acaba de rechazar la petición con identificador """ + str(idReq.encode('utf-8')) + """, por tanto no se ha generado el certificado digital.</p>
		</div>
		<ul>
			<table width="100%" border="0" align="center" cellpadding="0" cellspacing="0" id="idCA">
			<tbody>
			<tr>
				<td><div align="left"><a href="../web/tratReq.html"><img src="../web/images/icons/table_edit.png" width="16" height="16" /> Volver a Gestión de Peticiones</a></div></td>
				<td><div align="right"><a href='../web/index.html'/>Ir a Inicio <img src="../web/images/icons/house.png" width="16" height="16" /></a></div></td>

				</tr>
			</tbody>
			</table>
		<div class="section_break"></div>
		</ul>
		</form>
	<div id="footer">
		<p align="center">Creado por Francisco José González Troyano para la Asignatura Proyecto Informático de la <a href="http://www.informatica.us.es">ETSII</a></p>
	</div>
	</div>

	<img id="bottom" src="../web/images/bottom.png" alt="">
</body>
</html>""", 'utf-8')
	print htmlCode.encode("utf-8")



if __name__ == '__main__':
	certif = Certificado()
	print "Content-Type: text/html\n"     # sigue HTML

	if certif.nuevoEstado == "rechazada":	# petición rechazada
		# TODO html con respuesta del rechazo
		generaRechPetHtml(certif.idReq)

	else:
		xCert = lib.miarc_xml.XMLCert(certif.idCA)
		lib.miarc_xml.XMLCert.actualiza (xCert, certif)

		crearCertHtml (certif)
		generaRespHtml(certif)
