#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

import cgi
import cgitb; cgitb.enable()
from M2Crypto import X509
import crl
import os
import os.path
from types import *
import _xml
import lib.miarc_util

import sys
import traceback

class CFormulario:

	def __init__(self):
		"""Parsea el formulario con la info para generar la CRL.

		@return: objeto CFormulario
		"""
		form = cgi.FieldStorage(keep_blank_values=1) # parsea la petición, form se comporta como un diccionario
		# Identificador de la CA
		self.idCA = form["idCA"].value
		# Algoritmo de Firma
		self.firma = form["firma"].value

		# Fecha de próxima actualización - parámetro opcional
		# FR-dia
		self.frd = -1
		if form.has_key("fechaProxAct_1") and form["fechaProxAct_1"].value != "":
			self.frd = int(form["fechaProxAct_1"].value)
		# FR-mes
		self.frm = -1
		if form.has_key("fechaProxAct_2") and form["fechaProxAct_2"].value != "":
			self.frm = int(form["fechaProxAct_2"].value)
		#FR-anyo
		self.fra = -1
		if form.has_key("fechaProxAct_3") and form["fechaProxAct_3"].value != "":
			self.fra = int(form["fechaProxAct_3"].value)

		# Comprobando que se ha introducido una fecha y que es posterior 
		# a la actual. En el caso en que no se rellenen los tres campos de la 
		# fecha, se entiende que no desea establecer la fecha de la próxima 
		# actualización de la CRL.
		self.error = []
		if form.has_key("fechaProxAct_1") and form.has_key("fechaProxAct_2") and form.has_key("fechaProxAct_3") and form["fechaProxAct_1"].value != "" and form["fechaProxAct_2"].value != "" and form["fechaProxAct_3"].value != "":
			fechas = lib.miarc_util.posFechaAct (self.frd, self.frm, self.fra)
			if not fechas:
				self.error.append(1)		#"ERROR: La fecha introducida en el formulario no es posterior a la actual."

		# Comprobando que si se ha rellenado alguno de los tres campos de la 
		# fecha de próxima actualización, no se ha dejado ninguno vacío.
		if (form["fechaProxAct_1"].value != "" and (form["fechaProxAct_2"].value == "" or form["fechaProxAct_3"].value == "")) or (form["fechaProxAct_2"].value != "" and (form["fechaProxAct_1"].value == "" or form["fechaProxAct_3"].value == "")) or (form["fechaProxAct_3"].value != "" and (form["fechaProxAct_1"].value == "" or form["fechaProxAct_2"].value == "")):
			self.error.append(2)		#"ERROR: La fecha introducida en el formulario está incompleta."


	def gestErrores (self):
		""" Devuelve una cadena enumerando los errores detectados en el formulario.
		"""
		msg = ""
		if len(self.error) != 0:
			msg += """<br/>Este mensaje de error ha sido generado por el servidor de la aplicación, esto implica que existe algún problema con la verificación de los datos en el cliente.<br/>

<br/>El problema puede deberse a varias causas, por favor compruebe que cumple los siguientes requerimientos:
	<blockquote><br/>1.- Asegúrese de utilizar un navegador web que permita ejecutar codigo javascript.
	<br/>2.- No trate de realizar peticiones al servidor sin usar los formularios suministrados y no altere su código fuente.
	<br/>3.- Compruebe que ha rellenado correctamente el formulario, principalmente los campos marcados con *.</blockquote>

<br/>A continuación se muestran los fallos detectados:<br/> 
"""
			for i in self.error:
				if i==1:
					msg += """<br/>ERROR (01DC): La fecha introducida en el campo: "Fecha de la próxima actualización" debe ser posterior a la actual. \n"""
				if i==2:
					msg += """<br/>ERROR (02DC): La fecha introducida en el campo: "Fecha de la próxima actualización" no está completa.\n"""
		return msg


	def web_passphrase_callback(self, p="", q=""):
		dirAct = os.getcwd()
		pathCAXML = dirAct + '/web/CAs/CAs.xml'
		objCAXML = _xml.XMLCA(pathCAXML)
		clave = objCAXML.getClave(self.idCA)
		return str(clave)


def generaRespHtml(idCA, listaRevocacion):
	""" Pagina html que se mostrará tras pulsar el botón de Descargar CRL.
	"""
	
	if type(listaRevocacion) is NoneType:
		if crl.existeCRL(idCA):
			dirAct = os.getcwd()
			pathCRL = dirAct + '/CAs/' + idCA + '/' + idCA + 'PEM.crl'
			crlObj = X509.load_crl(pathCRL)
			crlTexto = crlObj.as_text()
			html = unicode("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>miARC - Nuevo Lista de Revocación de Certificados Emitida con Éxito.</title>
	<link rel="stylesheet" type="text/css" href="../web/estiloInd.css" media="all">
	<script type="text/javascript" src="../web/view.js"></script>
</head>
<body id="main_body" >
	<img id="top" src="../web/images/top.png" alt="">
	<img src="../web/images/miarc.jpg" alt="miARC" border="0" usemap="#myARCLogoMap" id="myARCLogo">
	<map name="myARCLogoMap" id="myARCLogoMap"><area shape="rect" coords="21,2,243,59" href="../web/index.html" /></map>
	<div id="contenedor">
		<br>
		<form id="form_39443" class="appnitro">
		<div class="form_description">
			<h2>Lista de Revocación de Certificados Creada con Éxito.</h2>
			<p>A continuación puede ver la lista de revocación de certificados en formato texto, en la parte iniferior dispone del enlace para descargarla.</p>
		</div>
		<ul>
			<table width="100%" border="0" align="center" cellpadding="0" cellspacing="0" id="idCA">
			<tbody>
			<tr>
				<td><div align="left"><a href="../web/CAs/""" + idCA + """/crl.html"><img src="../web/images/icons/report.png" width="16" height="16" /> Volver a Lista de Revocación de Certificados</a></div></td>
				<td><div align="right"><a href='../web/index.html'/>Ir a Inicio <img src="../web/images/icons/house.png" width="16" height="16" /></a></div></td>
				</tr>
			</tbody>
			</table>
		<div class="section_break"></div>
		<li>
			<pre>""" + crlTexto + """</pre>
		</li>
		<div class="section_break"></div>
		<table width="100%" border="0" align="center">
		<tbody>
 		<tr>
			<td><div align="center"><a href='../../../CAs/""" + idCA + """/""" + idCA + """PEM.crl' class=""><img src="../web/images/icons/report_disk.png" width="16" height="16" /> Descargar Lista de Revocación de Certificados</a></div></td>
 		</tr>
		</tbody>
		</table>
		<div class="section_break"></div>
		</ul>
		</form>
		<div id="footer">
			<p align="center">Creado por Francisco José González Troyano para la Asignatura Proyecto Informático de la <a href="http://www.informatica.us.es">ETSII</a></p>
		</div>
	</div>
	<img id="bottom" src="../web/images/bottom.png" alt="">
</body>
</html>""", 'utf-8')
		else:
			html = unicode("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
</head>
<body>
	No existe CRL para esta CA.
</body>
</html>""", 'utf-8')
	else:
	# dando formato texto a la CRL
		crlTexto = listaRevocacion.crl.as_text()
		html = unicode("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>miARC - Nuevo Lista de Revocación de Certificados Emitida con Éxito.</title>
	<link rel="stylesheet" type="text/css" href="../web/estiloInd.css" media="all">
	<script type="text/javascript" src="../web/view.js"></script>
</head>
<body id="main_body" >
	<img id="top" src="../web/images/top.png" alt="">
	<img src="../web/images/miarc.jpg" alt="miARC" border="0" usemap="#myARCLogoMap" id="myARCLogo">
	<map name="myARCLogoMap" id="myARCLogoMap"><area shape="rect" coords="21,2,243,59" href="../web/index.html" /></map>
	<div id="contenedor">
		<br>
		<form id="form_39443" class="appnitro">
		<div class="form_description">
			<h2>Lista de Revocación de Certificados Creada con Éxito.</h2>
			<p>A continuación puede ver la lista de revocación de certificados en formato texto, en la parte iniferior dispone del enlace para descargarla.</p>
		</div>
		<ul>
			<table width="100%" border="0" align="center" cellpadding="0" cellspacing="0" id="idCA">
			<tbody>
			<tr>
				<td><div align="left"><a href="../web/CAs/""" + idCA + """/crl.html"><img src="../web/images/icons/report.png" width="16" height="16" /> Volver a Lista de Revocación de Certificados</a></div></td>
				<td><div align="right"><a href='../web/index.html'/>Ir a Inicio <img src="../web/images/icons/house.png" width="16" height="16" /></a></div></td>
				</tr>
			</tbody>
			</table>
		<div class="section_break"></div>
		<li>
			<pre>""" + crlTexto + """</pre>
		</li>
		<div class="section_break"></div>
		<table width="100%" border="0" align="center">
		<tbody>
 		<tr>
			<td><div align="center"><a href='../../../CAs/""" + idCA + """/""" + idCA + """PEM.crl' class=""><img src="../web/images/icons/report_disk.png" width="16" height="16" /> Descargar Lista de Revocación de Certificados</a></div></td>
 		</tr>
		</tbody>
		</table>
		<div class="section_break"></div>
		</ul>
		</form>
		<div id="footer">
			<p align="center">Creado por Francisco José González Troyano para la Asignatura Proyecto Informático de la <a href="http://www.informatica.us.es">ETSII</a></p>
		</div>
	</div>
	<img id="bottom" src="../web/images/bottom.png" alt="">
</body>
</html>""", 'utf-8')
	print html.encode("utf-8")


if __name__ == '__main__':
	print "Content-Type: text/html"     # sigue HTML
	print								# linea en blanco, fin de la cabecera
	sys.stderr = sys.stdout
	try:
		dirAct = os.getcwd()
		formInfo = CFormulario()
		# control de errores en el formulario
		cadErrores = formInfo.gestErrores()
		if len(cadErrores) == 0:
			pathCRL = dirAct + '/CAs/' + formInfo.idCA + '/' + formInfo.idCA + 'PEM.crl'
			if os.path.exists(pathCRL):		# si ya existe una CRL realizamos el proceso de actualización
				lrc = X509.load_crl(pathCRL)
				listaRevCerts = crl.CRL(lrc)
				if formInfo.frd != -1 and formInfo.frm != -1 and formInfo.fra != -1:
					crlNueva = listaRevCerts.actualizar(formInfo.idCA, formInfo.web_passphrase_callback, formInfo.firma, formInfo.frd, formInfo.frm, formInfo.fra)
				else:
					crlNueva = listaRevCerts.actualizar(formInfo.idCA, formInfo.web_passphrase_callback, formInfo.firma)
	#			generaRespHtml(formInfo.idCA, listaRevCerts)
	#			print "Se ha actualizado la CRL correctamente"
			else:							# si no existe CRL, la creamos vacía
				# creación de una CRL vacía
				if formInfo.frd != -1 and formInfo.frm != -1 and formInfo.fra != -1:	# se ha especificado fecha próxima actualización
					crlNueva = crl.crearCRLVacia (formInfo.idCA, formInfo.web_passphrase_callback, formInfo.firma, formInfo.frd, formInfo.frm, formInfo.fra)
				else:
					crlNueva = crl.crearCRLVacia (formInfo.idCA, formInfo.web_passphrase_callback, formInfo.firma)
	#			generaRespHtml(formInfo.idCA, crl)
	#			print "Esta CA no ha revocado ningún certificado, se ha creado una CRL vacía."
			generaRespHtml(formInfo.idCA, crlNueva)
		else:
			print cadErrores
	except:
		print "\n\n<PRE>"
		traceback.print_exc()
