#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

import cgi
import cgitb; cgitb.enable()
import lib.miarc_util

class CFormulario:
	def __init__(self):
		"""
		Mapeo y control de errores en el servidor del formulario de creación de una nueva CA desde cero.

		@rtype: tupla.
		@return: objeto tupla.
		El primer elemento de la tupla indica si el formulario tiene cumplimentados los campos obligatorios con valores válidos, 
			Si tupla[0]==False, tupla[1] es una cadena con un mensaje de error.
			Si tupla[0]==True, el resto de la tupla contiene los valores ingresados en los campos del formulario ordenados.
		"""

		self.error = []									# lista con los codigos de error detectados al parsear el contenido del formulario
		form = cgi.FieldStorage(keep_blank_values=1) 	# parsea la petición, form se comporta como un diccionario
		# "keep_blank_values=1" evita que FielStorage ignore aquellos campos que contengan cadenas vacias

		# Nombre Identificador de la CA	
		if form.has_key("idCA") and form["idCA"].value != "":
			if lib.miarc_util.contieneEspacios(form["idCA"].value):		# idCA no puede tener espacios
				self.error.append(9)
				self.idCA = ''		# para que no salte una excepción al encontrarse en la línea 39 que self.idCA no tiene ningún valor asignado
			else:
				self.idCA = form["idCA"].value
		else:
			self.error.append(1) 	#"ERROR: Falta la información referente al Nombre Identificador de la CA."
			self.idCA = ''			# para que no salte una excepción al encontrarse en la línea 39 que self.idCA no tiene ningún valor asignado
		
		# Nombre Común (CN)
		if form.has_key("nombreCom") and form["nombreCom"].value != "":
			self.cn = form["nombreCom"].value
		else: 						# si no se le da un nombre común tomará por defecto el del nombre identificador
			self.cn = self.idCA

		# Apellido (SN)
		if form.has_key("apellido"):
			self.sn = form["apellido"].value
		
		# Sobrenombre (GN)
		if form.has_key("sobrenombre"):
			self.gn = form["sobrenombre"].value

		# Número de Serie
		if form.has_key("serial"):
			self.serial = form["serial"].value

		# Organización (O)
		if form.has_key("Organizacion") and form["Organizacion"].value != "":
			self.o = form["Organizacion"].value
		else:
			self.error.append(10) 			#ERROR: Introduzca una organización

		# Unidad Organizativa (OU)
		if form.has_key("unidadOrg"):
			self.ou = form["unidadOrg"].value

		# Email
		if form.has_key("email"):
			self.mail = form["email"].value

		# Localidad
		if form.has_key("direccion_3"):
			self.loc = form["direccion_3"].value

		# Provincia
		if form.has_key("direccion_4"):
			self.st = form["direccion_4"].value

		# País
		if form.has_key("direccion_6"):
			self.pais = form["direccion_6"].value

		# Número de Bits de la PrKey
		if form.has_key("numBitsPK") and form["numBitsPK"].value != "":
			opcBits = form["numBitsPK"].value
			numBits = ('512', '1024', '2048', '3072', '4096')
			self.nBits = numBits[int(opcBits)-1]
		else:
			self.nBits = '2048' 			# valor por defecto
	
		# Exponente de Key
		if form.has_key("exponente") and form["exponente"].value != "":
			opcExp = form["exponente"].value
			exps = ("RSA_3 (0x03L)", "RSA_F4 (0x10001L)")
			self.exp = exps[int(opcExp)-1]
		else:
			self.exp = "RSA_F4 (0x10001L)" 	# valor por defecto

		# Password
		if form.has_key("password") and form["password"].value != "":
			self.passw = form["password"].value
		else:
			self.error.append(2) 			#"ERROR: Introduzca un password para encriptar su clave RSA."

		# Verificación del password
		if form.has_key("vPassword") and form["vPassword"].value != "":
			vpassw = form["vPassword"].value
		else:
			self.error.append(3) 			#"ERROR: Rellene la casilla de verificación del password para encriptar su clave RSA."

		# Comprobando que coinciden las dos casillas de password
		if (form.has_key("password") and form["password"].value == "") or (form.has_key("vPassword") and form["vPassword"].value == "") or (self.passw != vpassw):
			self.error.append(4)			#"ERROR: Las casillas Password y Verificacion de Password NO coinciden."

		# Algoritmo de firma del certificado
		if form.has_key("firma") and form["firma"].value != "":
			self.firma = form["firma"].value
		else:
			self.firma = 'sha1'

		# Fecha de Revocación
		# FR-dia
		if form.has_key("fechaCaduc_1") and form["fechaCaduc_1"].value != "":
			self.frd = form["fechaCaduc_1"].value
		else:
			self.error.append(5) 			#"ERROR: Seleccione el día en la fecha de revocación del certificado."
		# FR-mes
		if form.has_key("fechaCaduc_2") and form["fechaCaduc_2"].value != "":
			self.frm = form["fechaCaduc_2"].value
		else:
			self.error.append(6) 			#"ERROR: Seleccione el mes en la fecha de revocación del certificado."
		#FR-anyo
		if form.has_key("fechaCaduc_3") and form["fechaCaduc_3"].value != "":
			self.fra = form["fechaCaduc_3"].value
		else:
			self.error.append(7) 			#"ERROR: Seleccione el año en la fecha de revocación del certificado."

		# Comprobando que la fecha de caducidad es posterior a la actual.
		if form.has_key("fechaCaduc_1") and form.has_key("fechaCaduc_2") and form.has_key("fechaCaduc_3") and form["fechaCaduc_1"].value != "" and form["fechaCaduc_2"].value != "" and form["fechaCaduc_3"].value != "":
			fechas = lib.miarc_util.posFechaAct (int(self.frd), int(self.frm), int(self.fra))
			if not fechas:
				self.error.append(8)		#"ERROR: La fecha introducida en el formulario no es posterior a la actual."


	def gestErrores (self):
		""" 
		Devuelve una cadena enumerando los errores detectados en el formulario.
		"""
		msg = ""
		if len(self.error) != 0:
			msg += """<br/>Este mensaje de error ha sido generado por el servidor de la aplicación, esto implica que existe algún problema con la verificación de los datos en el cliente.<br/>

<br/>El problema puede deberse a varias causas, por favor compruebe que cumple los siguientes requerimientos:
	<blockquote><br/>1.- Asegúrese de utilizar un navegador web que permita ejecutar código javascript.
	<br/>2.- No trate de realizar peticiones al servidor sin usar los formularios suministrados y no altere su código fuente.
	<br/>3.- Compruebe que ha rellenado correctamente el formulario, principalmente los campos marcados con *.</blockquote>

<br/>A continuación se muestran los fallos detectados:<br/> 
"""
			for i in self.error:
				if i==1:
					msg += """<br/>ERROR (01NCA): Debe rellenar el campo: "*Nombre Identificador de la CA". \n"""
				if i==2:
					msg += """<br/>ERROR (02NCA): Debe rellenar el campo: "*Password". \n"""
				if i==3:
					msg += """<br/>ERROR (03NCA): Debe rellenar el campo: "*Verificación del Password". \n"""
				if i==4:
					msg += """<br/>ERROR (04NCA): Las casillas "*Password" y "*Verificación del Password" NO coinciden. \n"""
				if i==5:
					msg += """<br/>ERROR (05NCA): Debe rellenar el campo: "*Fecha de Caducidad - DD". \n"""
				if i==6:
					msg += """<br/>ERROR (06NCA): Debe rellenar el campo: "*Fecha de Caducidad - MM". \n"""
				if i==7:
					msg += """<br/>ERROR (07NCA): Debe rellenar el campo: "*Fecha de Caducidad - AAAA". \n"""
				if i==8:
					msg += """<br/>ERROR (08NCA): La fecha introducida en el campo "*Fecha de Caducidad" debe ser posterior a la actual. \n"""
				if i==9:
					msg += """<br/>ERROR (09NCA): El campo "*Nombre Identificador de la CA" no admite espacios. \n"""
				if i==10:
					msg += """<br/>ERROR (10NCA): ebe rellenar el campo: "*Organización". \n"""
		return msg


	def vistaPrevia(self):
		""" 
		Genera una página html con el contenido seleccionado para que el usuario compruebe los valores elegidos.
		
		La página contiene campos <input type="hidden"> manteniendo un formulario con la info suministrada oculta al usuario, 
		permitiéndonos parsearlo fácilmente mediante cgi.FieldStorage para generar el certificado raíz.
		Esta funcion es llamada unicamente en el caso en que el formulario y su contenido sean correctos.
		"""
		# generamos la vista previa html
		htmlCodeIni = unicode ("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>miARC - Información suministrada para configurar la nueva CA.</title>
	<link rel="stylesheet" type="text/css" href="../web/style.css" media="all">
	<script type="text/javascript" src="../web/view.js"></script>
	<script>
	function mostrar(enla) {
	  obj = document.getElementById('oculto');
	  obj.style.visibility = (obj.style.visibility == 'hidden') ? 'visible' : 'hidden';
	  enla.innerHTML = (enla.innerHTML == 'Ocultar Password') ? 'Mostrar Password' : 'Ocultar Password';
	}
	</script>
</head>
<body id="main_body" >
	<img id="top" src="../web/images/top.png" alt="">
	<img src="../web/images/miarc.jpg" alt="miARC" border="0" usemap="#myARCLogoMap" id="myARCLogo">
	<map name="myARCLogoMap" id="myARCLogoMap"><area shape="rect" coords="21,2,243,59" href="../web/index.html" /></map>
	<div id="form_container">
	<br>
	<form id="form_39443" class="appnitro" method="post" action="http://localhost:8000/cgi-bin/crearNuevaCA.py">
		<div class="form_description">
		<h2>Nueva Autoridad de Certificación.</h2>
		<p>A continuación se muestra la información que ha ingresado para crear la nueva autoridad de certificación, antes de continuar compruebe que todos los datos son correctos.</p>
		</div>
		<ul>
			<li id="li_1" >
				<label class="description" for="idCA">Nombre Identificador de la CA: </label>
				<div>
					<input type=hidden id="idCA" name="idCA" class="element text medium" type="text" maxlength="255" value='""" + self.idCA + """'/>
					""" + self.idCA + """
				</div>
			</li>
			<li class="section_break"></li>""", 'utf-8')

		cnCode = unicode ("""
			<li id="li_3" >
				<label class="description" for="nombreCom">Nombre Común (CN): </label>
				<div>
					<input type=hidden id="nombreCom" name="nombreCom" class="element text medium" type="text" maxlength="255" value='""" + self.cn + """'/>
					""" + self.cn + """
				</div>
			</li>""", 'utf-8')

		snCode = unicode ("""
			<li id="li_4" >
				<label class="description" for="apellido">Apellido (SN): </label>
				<div>
					<input type=hidden id="apellido" name="apellido" class="element text medium" type="text" maxlength="255" value='""" + self.sn + """'/> 
					""" + self.sn + """
				</div>
			</li>""", 'utf-8')

		gnCode = unicode ("""
			<li id="li_5" >
				<label class="description" for="sobrenombre">Sobrenombre (GN): </label>
				<div>
					<input type=hidden id="sobrenombre" name="sobrenombre" class="element text medium" type="text" maxlength="255" value='""" + self.gn + """'/>
					""" + self.gn + """
				</div>
			</li>""", 'utf-8')

		nsCode = unicode ("""
			<li id="li_9" >
				<label class="description" for="serial">Número de Serie: </label>
				<div>
					<input type=hidden id="serial" name="serial" class="element text medium" type="text" maxlength="255" value='""" + self.serial + """'/>
					""" + self.serial + """
				</div>
			</li>""", 'utf-8')

		oCode = unicode ("""
			<li id="li_6" >
				<label class="description" for="organizacion">Organización (O): </label>
				<div>
					<input type=hidden id="organizacion" name="organizacion" class="element text large" type="text" maxlength="255" value='""" + self.o + """'/>
					""" + self.o + """
				</div>
			</li>""", 'utf-8')

		ouCode = unicode ("""
			<li id="li_7" >
				<label class="description" for="unidadOrg">Unidad Organizativa (OU): </label>
				<div>
					<input type=hidden id="unidadOrg" name="unidadOrg" class="element text large" type="text" maxlength="255" value='""" + self.ou + """'/>
					""" + self.ou + """
				</div>
			</li>""", 'utf-8')

		emailCode = unicode ("""
			<li id="li_10" >
				<label class="description" for="email">E-mail: </label>
				<div>
					<input type=hidden id="email" name="email" class="element text large" type="text" maxlength="25" value='""" + self.mail + """'/>
					""" + self.mail + """
				</div>
			</li>""", 'utf-8')

		dirCode = unicode ("""
			<li id="li_8" >
				<label class="description" for="direccion">Datos de Localización: </label>
			</li>""", 'utf-8')

		locCode = unicode ("""
			<li id="li_10" >
				<label class="description" for="direccion_3">Localidad: </label>
				<div>
					<input type=hidden id="direccion_3" name="direccion_3" class="element text medium" value='""" + self.loc + """' type="text"/>
					""" + self.loc + """
				</div>
			</li>""", 'utf-8')

		estCode = unicode ("""
			<li id="li_10" >
				<label class="description" for="direccion_4">Estado / Provincia / Región: </label>
				<div>
					<input type=hidden id="direccion_4" name="direccion_4" class="element text medium" value='""" + self.st + """' type="text"/>
					""" + self.st + """
				</div>
			</li>""", 'utf-8')

		paisCode = unicode ("""
			<li id="li_10" >
				<label class="description" for="direccion_6">País: </label>
				<div>
					<input type=hidden id="direccion_6" name="direccion_6" class="element text medium" value='""" + self.pais + """' type="text"/>
					""" + self.pais + """
				</div>
			</li>""", 'utf-8')

		htmlCodeFin = unicode ("""
			<li class="section_break"></li>
			<li id="li_12" >
				<label class="description" for="numBitsPK">Número de bits de la Clave Privada: </label>
				<div>
					<input type=hidden id="numBitsPK" name="numBitsPK" class="element text medium" value='""" + self.nBits + """' type="text"/>
					""" + self.nBits + """
				</div>
			</li>
			<li id="li_13" >
				<label class="description" for="exponente">Exponente: </label>
				<div>
					<input type=hidden id="exponente" name="exponente" class="element text medium" value='""" + self.exp + """' type="text"/>
					""" + self.exp + """
				</div>
			</li>
			<li id="li_14" >
				<input type=hidden id="password" name="password" class="element text large" type="password" maxlength="255" value='""" + self.passw + """'/>
				<label class="description" for="password">Password: </label>
				Por razones de seguridad el password se mantiene oculto por defecto, si desea verlo pulse sobre el siguiente enlace: <a href="#" onclick="mostrar(this); return false" />Mostrar Password</a>.
				<div id="oculto" style="visibility:hidden">
					Password: """ + self.passw + """
				</div>
			</li>
			<li id="li_15" >
				<label class="description" for="firma">Algoritmo de Firma: </label>
				<div>
					<input type=hidden id="firma" name="firma" class="element text large" type="firma" maxlength="255" value='""" + self.firma + """'/>
					""" + self.firma + """
				</div>
			</li>
			<li class="section_break"></li>
			<li id="li_17" >
				<label class="description" for="fechaCaduc">Fecha de Revocación: </label>
				<div>
					<input type=hidden id="fechaCaduc_1" name="fechaCaduc_1" class="element text" size="2" maxlength="2" value='""" + self.frd + """' type="text"/>
					<input type=hidden id="fechaCaduc_2" name="fechaCaduc_2" class="element text" size="2" maxlength="2" value='""" + self.frm + """' type="text"/>
					<input type=hidden id="fechaCaduc_3" name="fechaCaduc_3" class="element text" size="2" maxlength="2" value='""" + self.fra + """' type="text"/>
					""" + self.frd + '/' + self.frm + '/' + self.fra + """
				</div>
			</li>
			<li class="buttons">
				<input type="hidden" name="form_id" value="39443"/>
				<input id="backForm" class="button_text" type="button" name="submit" value="Cambiar" onclick="history.back()"/>
				<input id="saveForm" class="button_text" type="submit" name="submit" value="Continuar"/>
			</li>
		</ul>
	</form>
	</div>
	<img id="bottom" src="../web/images/bottom.png" alt="">
</body>
</html>""", 'utf-8')
		# Generación del código a imprimir
		html = htmlCodeIni
		if self.cn != '':
			html += cnCode
		if self.sn != '':
			html += snCode
		if self.gn != '':
			html += gnCode
		if self.serial != '':
			html += nsCode
		if self.o != '':
			html += oCode
		if self.ou != '':
			html += ouCode
		if self.mail != '':
			html += emailCode
		if self.loc != '' or self.st != '' or self.pais != '':
			html += dirCode
		if self.loc != '':
			html += locCode
		if self.st != '':
			html += estCode
		if self.pais != '':
			html += paisCode
		html += htmlCodeFin
		print html.encode('utf-8')

if __name__ == '__main__':
	print "Content-Type: text/html"     # sigue HTML
	print								# linea en blanco, fin de la cabecera
	cf = CFormulario()
	cadErrores = cf.gestErrores()
	if len(cadErrores) == 0:			# si no hay errores
		cf.vistaPrevia()				# mostrar la vista previa
	else:
		print cadErrores				# mostrar errores
